/*
    dlggpureftpdimpl.cpp

    Copyright (c) 2006 by Xavier Facq

    gpureftpd (c) 2006 by the gpureftpd developers

    *************************************************************************
    *                                                                       *
    * This program is free software; you can redistribute it and/or modify  *
    * it under the terms of the GNU General Public License as published by  *
    * the Free Software Foundation; either version 2 of the License, or     *
    * (at your option) any later version.                                   *
    *                                                                       *
    *************************************************************************
*/
/*

TODO list:

- delete unused files when anonymous or when anonymous not allowed if needed etc...
- split format / file for altlog ?
- check available for syslogfacilities options or leave it free ?
- set the focus to the bad IP widget in verifyIP ?

*/


#include "global.h"
#include "dlggpureftpdimpl.h"
#include "dlgvirtualusersimpl.h"

#include <qmessagebox.h>
#include <qcheckbox.h>
#include <qfile.h>
#include <qlineedit.h>
#include <qspinbox.h>
#include <qbuttongroup.h>
#include <qcombobox.h>
#include <qfiledialog.h>
#include <qpushbutton.h>
#include <qlistview.h>
#include <qprogressdialog.h>
#include <qregexp.h>
#include <qobjectlist.h>
#include <qobject.h>
#include <qprocess.h>

/*
	@constructor
	Read the configuration files and feel the form at the initialisation.
*/
DlgGPureftpdImpl::DlgGPureftpdImpl(QWidget *parent, const char *name)
    :DlgGPureftpd(parent, name)
{
	QString s;
	QProgressDialog *progression;

	progression = new QProgressDialog(this, "Reading configuration...");
 	progression->setAutoClose(TRUE);
 	progression->setProgress(0);

	s = QString::null;
	m_SomethingHaveChanged = FALSE;

	QFileInfo fileinfo(PUREFTPD_PATH);
	m_UserCanSave = fileinfo.isWritable();

	// server tab
	s = readOptionFile("Bind");
	if(s != QString::null && s.find(',') != -1)
	{
		leIPAdress->setText(s.section(",",0,0));
		lePort->setText(s.section(",",1,1));
	}

	s = readOptionFile("ForcePassiveIP");
	if(s != QString::null)
	{
		leForcePassiveIP->setText(s);
		s = readOptionFile("PassivePortRange");
		if(s != QString::null)
		{
			cbEnablePassiveMode->setChecked(TRUE);
			lePassivePortRange->setText(s);
		}
	}
	else
	{
		cbEnablePassiveMode->setChecked(FALSE);
		leForcePassiveIP->setDisabled(TRUE);
		lePassivePortRange->setDisabled(TRUE);
	}

 	sbMaxClientsNumber->setValue(getIntValue("MaxClientsNumber", 50));
 	sbMaxClientsPerIP->setValue(getIntValue("MaxClientsPerIP", 0));
 	sbMaxIdleTime->setValue(getIntValue("MaxIdleTime", 15));
 	spUserBandwidth->setValue(getIntValue("UserBandwidth", 0));
	cbDaemonize->setChecked(isOptionActive("Daemonize", FALSE));
	DontResolve->setChecked(isOptionActive("DontResolve", FALSE));
	BrokenClientsCompatibility->setChecked(isOptionActive("BrokenClientsCompatibility", FALSE));

    connect(leIPAdress, SIGNAL(lostFocus()), this, SLOT(verifyIpAdress()));
    connect(leForcePassiveIP, SIGNAL(lostFocus()), this, SLOT(verifyIpAdress()));

	if(m_UserCanSave)
	{
	    connect(pbVirtualUser, SIGNAL(clicked()), this, SLOT(manageVirtualUsers()));
		lblWarningStatus->setText("");
	}
	else
	{
		pbVirtualUser->setDisabled(TRUE);
		lblWarningStatus->setText("Note: Changes will not be saved (not allowed).");
	}

 	progression->setProgress(20);

	// Authentication tab
	AnonymousOnly->setChecked(isOptionActive("AnonymousOnly", FALSE));
	s = readOptionFile("PureDB");
	if(s != QString::null) lePureDB->setText(s);
	cbUnixAuthentication->setChecked(isOptionActive("UnixAuthentication", FALSE));
	cbPAMAuthentication->setChecked(isOptionActive("PAMAuthentication", FALSE));

	cbTLS->insertItem("Disable");
	cbTLS->insertItem("All");
	cbTLS->insertItem("SSL/TLS only");

	cbTLS->setCurrentItem(getIntValue("TLS", 0));

	cbAuthMethod->insertItem("ExtAuth");
	cbAuthMethod->insertItem("LDAP");
	cbAuthMethod->insertItem("MySQL");
	cbAuthMethod->insertItem("PGSQL");

	lvAuthMethodList->setColumnWidthMode (1, QListView::Maximum);

	QListViewItem *qlv;

	s = readOptionFile("ExtAuth");
	if(s != QString::null)
	{
		qlv = new QListViewItem(lvAuthMethodList, "ExtAuth", s);
		lvAuthMethodList->insertItem(qlv);
	}

	s = readOptionFile("LDAPConfigFile");
	if(s != QString::null)
	{
		qlv = new QListViewItem(lvAuthMethodList, "LDAP", s);
		lvAuthMethodList->insertItem(qlv);
	}

	s = readOptionFile("MySQLConfigFile");
	if(s != QString::null)
	{
		qlv = new QListViewItem(lvAuthMethodList, "MySQL", s);
		lvAuthMethodList->insertItem(qlv);
	}

	s = readOptionFile("PGSQLConfigFile");
	if(s != QString::null)
	{
		qlv = new QListViewItem(lvAuthMethodList, "PGSQL", s);
		lvAuthMethodList->insertItem(qlv);
	}

	setAllAuthenticateOptionsDisabled(AnonymousOnly->isChecked());

    connect(pbPureDBBrowse, SIGNAL(clicked()), this, SLOT(selectFile()));
    connect(pbPathToAuthMethod, SIGNAL(clicked()), this, SLOT(selectFile()));
    connect(pbAddAuthMethod, SIGNAL(clicked()), this, SLOT(addAuthMethod()));
    connect(pbRemoveAuthMethod, SIGNAL(clicked()), this, SLOT(removeAuthMethod()));
    connect(pbUpdate, SIGNAL(clicked()), this, SLOT(updateAuthMethod()));
    connect(AnonymousOnly, SIGNAL(toggled(bool)), this, SLOT(setAllAuthenticateOptionsDisabled(bool)));

 	progression->setProgress(40);

	// Advanced tab
	cbNATmode->setChecked(isOptionActive("NATmode", FALSE));
 	CallUploadScript->setChecked(isOptionActive("CallUploadScript", FALSE));
 	CustomerProof->setChecked(isOptionActive("CustomerProof", FALSE));
 	sbMaxDiskUsage->setValue(getIntValue("MaxDiskUsage", 0));
	s = readOptionFile("FortunesFile");
	if(s != QString::null) leFortunesFile->setText(s);

	if(isOptionActive("IPV4Only", FALSE))
		bgIPVersion->setButton(1);
	else if(isOptionActive("IPV6Only", FALSE))
		bgIPVersion->setButton(2);
	else
		bgIPVersion->setButton(0);

    connect(pbFortuneFileBrowse, SIGNAL(clicked()), this, SLOT(selectFile()));

 	progression->setProgress(55);

	// Users options tab
	cbCreateHomeDir->setChecked(isOptionActive("CreateHomeDir", FALSE));
	cbNoChmod->setChecked(isOptionActive("NoChmod", FALSE));
 	ChrootEveryone->setChecked(isOptionActive("ChrootEveryone", FALSE));
 	spMinUID->setValue(getIntValue("MinUID", 500));
 	sbTrustedGID->setValue(getIntValue("TrustedGID", 500));
 	AllowUserFXP->setChecked(isOptionActive("AllowUserFXP", FALSE));

	cbAntiWarez->setChecked(isOptionActive("AntiWarez", FALSE));
 	NoAnonymous->setChecked(isOptionActive("NoAnonymous", FALSE));
	AllowAnonymousFXP->setChecked(isOptionActive("AllowAnonymousFXP", FALSE));
	AnonymousCanCreateDirs->setChecked(isOptionActive("AnonymousCanCreateDirs", FALSE));
	AnonymousCantUpload->setChecked(isOptionActive("AnonymousCantUpload", FALSE));
	s = readOptionFile("AnonymousRatio");
	if(s != QString::null) lnAnonymousRatio->setText(s);
 	sbAnonymousBandwidth->setValue(getIntValue("AnonymousBandwidth", 0));
	s = readOptionFile("TrustedIP");
	if(s != QString::null) leTrustedIP->setText(s);

	setAllAnonymousOptionsDisabled(NoAnonymous->isChecked());

    connect(NoAnonymous, SIGNAL(toggled(bool)), this, SLOT(setAllAnonymousOptionsDisabled(bool)));
    connect(leTrustedIP, SIGNAL(lostFocus()), this, SLOT(verifyIpAdress()));

 	progression->setProgress(70);

	// logging tab
	LogPID->setChecked(isOptionActive("LogPID", FALSE));
	VerboseLog->setChecked(isOptionActive("VerboseLog", FALSE));

	s = readOptionFile("AltLog");
	if(s != QString::null) leAltLog->setText(s);

	s = readOptionFile("SyslogFacility");
	if(s == QString::null) s = "ftp";
	leSyslogFacility->setText(s);

 	progression->setProgress(85);

	// Files & Directories tab
	s = readOptionFile("LimitRecursion");
	if(s != QString::null) leLimitRecursion->setText(s);
	s = readOptionFile("PerUserLimits");
	if(s != QString::null) lePerUserLimits->setText(s);
	s = readOptionFile("Quota");
	if(s != QString::null) leQuota->setText(s);
	s = readOptionFile("UserRatio");
	if(s != QString::null) leUserRatio->setText(s);
	s = readOptionFile("MaxLoad");
	if(s != QString::null) leMaxLoad->setText(s);
	s = readOptionFile("Umask");
	if(s != QString::null) leUmask->setText(s);
	AutoRename->setChecked(isOptionActive("AutoRename", FALSE));
	NoRename->setChecked(isOptionActive("NoRename", FALSE));
	KeepAllFiles->setChecked(isOptionActive("KeepAllFiles", FALSE));
	cbDisplayDotFiles->setChecked(isOptionActive("DisplayDotFiles", FALSE));
	cbAllowDotFiles->setChecked(isOptionActive("AllowDotFiles", FALSE));
	cbProhibitDotFilesWrite->setChecked(isOptionActive("ProhibitDotFilesWrite", FALSE));
	cbProhibitDotFilesRead->setChecked(isOptionActive("ProhibitDotFilesRead", FALSE));

	QObject *obj;
	QObjectList *widgets = queryList();
	QObjectListIterator it(*widgets);
	
	while((obj = it.current()) != NULL)
	{
		QString className = obj->className();

		if(className == "QLineEdit" || className == "QComboBox")
		    connect(obj, SIGNAL(textChanged(const QString&)), this, SLOT(somethingChangeSomewhere(const QString&)));
		else if(className == "QSpinBox")
		    connect(obj, SIGNAL(valueChanged(const QString&)), this, SLOT(somethingChangeSomewhere(const QString&)));
 		else if(className == "QCheckBox" || className == "QRadioButton")
 		    connect(obj, SIGNAL(toggled(bool)), this, SLOT(somethingChangeSomewhere(bool)));

		++it;
	}

 	progression->setProgress(100);

	setMinimumSize(width(), height());
	setMaximumSize(width(), height());
}

/*
	Read the "optionName" file and return TRUE if the option is activated
*/
bool DlgGPureftpdImpl::isOptionActive(QString optionName, bool bDefaultValue)
{
	QString s;
	bool bResult;

	bResult = bDefaultValue;
	s = readOptionFile(optionName);
	if(s != QString::null) bResult = toBoolean(s);

	return bResult;
}

int DlgGPureftpdImpl::getIntValue(QString optionName, int nDefaultValue)
{
	QString s;
	int nResult;

	nResult = nDefaultValue;
	s = readOptionFile(optionName);
	if(s != QString::null) nResult = s.toInt();

	return nResult;
}

bool DlgGPureftpdImpl::toBoolean(QString optionName)
{
	bool b;
	QString s;

	b = FALSE;
	s = optionName.stripWhiteSpace().lower(); 
	if(s == "yes" || s == "1" || s == "on") b = TRUE;

	return b;
}

QString DlgGPureftpdImpl::readOptionFile(QString optionName)
{
	QString s;
	QString sfilename;

	s = QString::null;
	sfilename = PUREFTPD_CONFIG_PATH + optionName;

	if(QFile::exists(sfilename))
	{
		QFile f(sfilename);
		if (f.open( IO_ReadOnly))
		{
			QTextStream ts( &f );
			s = ts.read();
		}
	}

	return s;
}

/*
	Open a file dialog box and feel the associated lineEdit 
	to the button with the selected file.
*/
void DlgGPureftpdImpl::selectFile()
{
	QString name;
	QLineEdit *le;
	const QPushButton *pb;

	QFileDialog dlg(PUREFTPD_PATH, "", 0, 0, TRUE);
	dlg.setFont(font());
	dlg.setFilter("All files (*)");

	if (dlg.exec() == QDialog::Accepted)
	{
		pb = dynamic_cast<const QPushButton *>(QObject::sender());
		name = QString(pb->name()).lower();

		if(name == "pbpuredbbrowse")
			le = lePureDB;
		else if(name == "pbpathtoauthmethod")
			le = lePathToAuthMethod;
		else if(name == "pbfortunefilebrowse")
			le = leFortunesFile;
		else
			le = NULL;

		if(le != NULL)
			le->setText(dlg.selectedFile());
	}
}

void DlgGPureftpdImpl::addAuthMethod()
{
	if(!cbAuthMethod->currentText().isEmpty() && !lePathToAuthMethod->text().isEmpty())
	{
		if(lvAuthMethodList->findItem(cbAuthMethod->currentText(), 0) != NULL)
		   messageBox("Information", "An authentication method for \"" + cbAuthMethod->currentText() + "\" allready exist.", QMessageBox::Information);
		else
		{
			QListViewItem qlv = new QListViewItem(lvAuthMethodList, cbAuthMethod->currentText(), lePathToAuthMethod->text());
			lvAuthMethodList->insertItem(&qlv);
			m_SomethingHaveChanged = TRUE;
		}
	}
}

void DlgGPureftpdImpl::removeAuthMethod()
{
	if(lvAuthMethodList->selectedItem() != 0)
	{
		lvAuthMethodList->takeItem (lvAuthMethodList->selectedItem());
		m_SomethingHaveChanged = TRUE;
	}
}

void DlgGPureftpdImpl::updateAuthMethod()
{
	if(lvAuthMethodList->selectedItem() != 0)
	{
		if(!cbAuthMethod->currentText().isEmpty() && !lePathToAuthMethod->text().isEmpty())
		{
			if(lvAuthMethodList->findItem(cbAuthMethod->currentText(), 0) != NULL)
				messageBox("Information", "An authentication method for \"" + cbAuthMethod->currentText() + "\" allready exist.", QMessageBox::Information);
			else
			{
				lvAuthMethodList->selectedItem()->setText(0, cbAuthMethod->currentText());
				lvAuthMethodList->selectedItem()->setText(1, lePathToAuthMethod->text());
				m_SomethingHaveChanged = TRUE;
			}
		}
	}
}

void DlgGPureftpdImpl::setAllAuthenticateOptionsDisabled(bool state)
{
	lblPureDB->setDisabled(state);
	lePureDB->setDisabled(state);
	cbUnixAuthentication->setDisabled(state);
	cbPAMAuthentication->setDisabled(state);
	lblTLS->setDisabled(state);
	cbTLS->setDisabled(state);
	lblAuthMethod->setDisabled(state);
	lblPathToAuthMethod->setDisabled(state);
	lePathToAuthMethod->setDisabled(state);
	cbAuthMethod->setDisabled(state);
    pbPureDBBrowse->setDisabled(state);
    pbPathToAuthMethod->setDisabled(state);
    pbAddAuthMethod->setDisabled(state);
    pbRemoveAuthMethod->setDisabled(state);
    pbUpdate->setDisabled(state);
	lvAuthMethodList->setDisabled(state);
}

void DlgGPureftpdImpl::setAllAnonymousOptionsDisabled(bool state)
{
	cbAntiWarez->setDisabled(state);
	AllowAnonymousFXP->setDisabled(state);
	AnonymousCanCreateDirs->setDisabled(state);
	AnonymousCantUpload->setDisabled(state);
	lblAnonymousRatio->setDisabled(state);
	lnAnonymousRatio->setDisabled(state);
	lblAnonymousBandWidth->setDisabled(state);
	sbAnonymousBandwidth->setDisabled(state);
	lblTrustedIP->setDisabled(state);
	leTrustedIP->setDisabled(state);
}

bool DlgGPureftpdImpl::verifyIpAdress()
{
/*
	// I disabled it a moment cause it's not very userfull !
	bool b;
	const QLineEdit *le;

	b = FALSE;
	le = dynamic_cast<const QLineEdit *>(QObject::sender());

	if(le != NULL && le->text() != QString::null)
	{
		QRegExp regexp(QString::fromLatin1("^(25[0-5]|2[0-4]\\d|[0-1]?\\d?\\d)(\\.(25[0-5]|2[0-4]\\d|[0-1]?\\d?\\d)){3}$"));
	
		if(regexp.search(le->text()) > -1)	
			b = TRUE;
		else
			messageBox("Error", "This is not a valid IP adress", QMessageBox::Information);
	}

	return b;
*/
	return TRUE;
}

void DlgGPureftpdImpl::somethingChangeSomewhere(const QString& newText)
{
	if(!m_SomethingHaveChanged) m_SomethingHaveChanged = TRUE;
}

void DlgGPureftpdImpl::somethingChangeSomewhere(bool b)
{
	if(!m_SomethingHaveChanged) m_SomethingHaveChanged = TRUE;
}

void DlgGPureftpdImpl::accept()
{
	if(!m_UserCanSave)
	{
		messageBox("Error", "You are not allowed to change the configuration.", QMessageBox::Warning);
		return;
	}
	else
	{
		if(m_SomethingHaveChanged && messageBox("Confirm save", "Do you want to save your changes ?", QMessageBox::Question, QMessageBox::Yes, QMessageBox::No) == QMessageBox::No)
			return;
	
		saveConfiguration();
	}

	DlgGPureftpd::accept();
}

void DlgGPureftpdImpl::reject()
{
	if(m_SomethingHaveChanged && m_UserCanSave && messageBox("Confirm quit", "Do you want to quit without saving your changes ?", QMessageBox::Question, QMessageBox::Yes, QMessageBox::No) == QMessageBox::No)
		return;

	DlgGPureftpd::reject();
}

bool DlgGPureftpdImpl::saveConfiguration()
{
	QString s;
	QProcess *proc;
	QProgressDialog *progression;

	progression = new QProgressDialog(this, "Writing configuration...");
 	progression->setAutoClose(TRUE);
 	progression->setProgress(0);

	// server tab
	if(!leIPAdress->text().isEmpty() && !lePort->text().isEmpty())
		writeFileOption(PUREFTPD_CONFIG_PATH, "Bind", leIPAdress->text() + "," + lePort->text());
	else
		removeFileOption(PUREFTPD_CONFIG_PATH, "Bind");

	if(cbEnablePassiveMode->isChecked() && 
	   !leForcePassiveIP->text().isEmpty() && !lePassivePortRange->text().isEmpty())
	{
		writeFileOption(PUREFTPD_CONFIG_PATH, "ForcePassiveIP", leForcePassiveIP->text());
		writeFileOption(PUREFTPD_CONFIG_PATH, "PassivePortRange", lePassivePortRange->text());
	}
	else
	{
		removeFileOption(PUREFTPD_CONFIG_PATH, "ForcePassiveIP");
		removeFileOption(PUREFTPD_CONFIG_PATH, "PassivePortRange");
	}

	if(sbMaxClientsNumber->text().toInt() != 0 && sbMaxClientsNumber->text().toInt() != 50)
		writeFileOption(PUREFTPD_CONFIG_PATH, "MaxClientsNumber", sbMaxClientsNumber->text());
	else
		removeFileOption(PUREFTPD_CONFIG_PATH, "MaxClientsNumber");

	if(sbMaxClientsPerIP->text().toInt() != 0)
		writeFileOption(PUREFTPD_CONFIG_PATH, "MaxClientsPerIP", sbMaxClientsPerIP->text());
	else
		removeFileOption(PUREFTPD_CONFIG_PATH, "MaxClientsPerIP");

	s = sbMaxIdleTime->text().section(" ", 0, 0);
	if(s.toInt() != 0 && s.toInt() != 15)
		writeFileOption(PUREFTPD_CONFIG_PATH, "MaxIdleTime", s);
	else
		removeFileOption(PUREFTPD_CONFIG_PATH, "MaxIdleTime");

	s = spUserBandwidth->text().section(" ", 0, 0);
	if(s.toInt() != 0)
		writeFileOption(PUREFTPD_CONFIG_PATH, "UserBandwidth", s);
	else
		removeFileOption(PUREFTPD_CONFIG_PATH, "UserBandwidth");

	proceedBoolOption("Daemonize", cbDaemonize->isChecked());
	proceedBoolOption("DontResolve", DontResolve->isChecked());
	proceedBoolOption("BrokenClientsCompatibility", BrokenClientsCompatibility->isChecked());

	// Authentication tab
	if(AnonymousOnly->isChecked())
	{
		writeFileOption(PUREFTPD_CONFIG_PATH, "AnonymousOnly", "yes");

		removeFileOption(PUREFTPD_CONFIG_PATH, "PureDB");
		removeFileOption(PUREFTPD_CONFIG_PATH, "UnixAuthentication");
		removeFileOption(PUREFTPD_CONFIG_PATH, "PAMAuthentication");
		removeFileOption(PUREFTPD_CONFIG_PATH, "TLS");

		removeFileOption(PUREFTPD_AUTH_PATH, "50pure");
		removeFileOption(PUREFTPD_AUTH_PATH, "60unix");
		removeFileOption(PUREFTPD_AUTH_PATH, "70pam");
	}
	else
	{
		removeFileOption(PUREFTPD_CONFIG_PATH, "AnonymousOnly");

		proceedStringOption("PureDB", lePureDB->text());
		if(lePureDB->text().isEmpty())
		{
			proc = new QProcess(this);
			proc->addArgument("rm");
			proc->addArgument(PUREFTPD_AUTH_PATH + "50pure");
			proc->start();
		}
		else
		{
			proc = new QProcess(this);
			proc->addArgument("ln");
			proc->addArgument("-s");
			proc->addArgument(PUREFTPD_CONFIG_PATH + "PureDB");
			proc->addArgument(PUREFTPD_AUTH_PATH + "50pure");
			proc->start();
		}

		proceedBoolOption("UnixAuthentication", cbUnixAuthentication->isChecked());
		if(cbUnixAuthentication->isChecked())
		{
			proc = new QProcess(this);
			proc->addArgument("ln");
			proc->addArgument("-s");
			proc->addArgument(PUREFTPD_CONFIG_PATH + "UnixAuthentication");
			proc->addArgument(PUREFTPD_AUTH_PATH + "60unix");
			proc->start();
		}
		else
		{
			proc = new QProcess(this);
			proc->addArgument("rm");
			proc->addArgument(PUREFTPD_AUTH_PATH + "60unix");
			proc->start();
		}

		proceedBoolOption("PAMAuthentication", cbPAMAuthentication->isChecked());
		if(cbPAMAuthentication->isChecked())
		{
			proc = new QProcess(this);
			proc->addArgument("ln");
			proc->addArgument("-s");
			proc->addArgument(PUREFTPD_CONFIG_PATH + "PAMAuthentication");
			proc->addArgument(PUREFTPD_AUTH_PATH + "70pam");
			proc->start();
		}
		else
		{
			proc = new QProcess(this);
			proc->addArgument("rm");
			proc->addArgument(PUREFTPD_AUTH_PATH + "70pam");
			proc->start();
		}

		if(cbTLS->currentItem() > 0)
			writeFileOption(PUREFTPD_CONFIG_PATH, "TLS", QString::number(cbTLS->currentItem()));
		else
			removeFileOption(PUREFTPD_CONFIG_PATH, "TLS");
	}
	
	removeFileOption(PUREFTPD_CONFIG_PATH, "ExtAuth");
	removeFileOption(PUREFTPD_CONFIG_PATH, "LDAPConfigFile");
	removeFileOption(PUREFTPD_CONFIG_PATH, "MySQLConfigFile");
	removeFileOption(PUREFTPD_CONFIG_PATH, "PGSQLConfigFile");

	QListViewItemIterator it(lvAuthMethodList);
    while (it.current())
	{
		QListViewItem *qlvi = it.current();

		if(qlvi != NULL)
		{
			s = qlvi->text(0);
	
			if(s == "ExtAuth")
				s = "ExtAuth";
			else if(s == "LDAP")
				s = "LDAPConfigFile";
			else if(s == "MySQL")
				s = "MySQLConfigFile";
			else if(s == "PGSQL")
				s = "PGSQLConfigFile";
			else
				s = QString::null;
	
			if(s != QString::null)
				writeFileOption(PUREFTPD_CONFIG_PATH, s, qlvi->text(1));
		}
		
        ++it;
	}

	// Advanced tab
	proceedBoolOption("CallUploadScript", CallUploadScript->isChecked());
	proceedBoolOption("CustomerProof", CustomerProof->isChecked());
	proceedBoolOption("NATmode", cbNATmode->isChecked());

	s = sbMaxDiskUsage->text().section(" ", 0, 0);
	if(s.toInt() != 0)
		writeFileOption(PUREFTPD_CONFIG_PATH, "MaxDiskUsage", s);
	else
		removeFileOption(PUREFTPD_CONFIG_PATH, "MaxDiskUsage");

	proceedStringOption("FortunesFile", leFortunesFile->text());

	if(bgIPVersion->selectedId() == 0)
	{
		removeFileOption(PUREFTPD_CONFIG_PATH, "IPV4Only");
		removeFileOption(PUREFTPD_CONFIG_PATH, "IPV6Only");
	}
	else if(bgIPVersion->selectedId() == 1)
	{
		writeFileOption(PUREFTPD_CONFIG_PATH, "IPV4Only", "yes");
		removeFileOption(PUREFTPD_CONFIG_PATH, "IPV6Only");
	}
	else if(bgIPVersion->selectedId() == 2)
	{
		writeFileOption(PUREFTPD_CONFIG_PATH, "IPV6Only", "yes");
		removeFileOption(PUREFTPD_CONFIG_PATH, "IPV4Only");
	}

	// Users options tab
	proceedBoolOption("CreateHomeDir", cbCreateHomeDir->isChecked());
	proceedBoolOption("NoChmod", cbNoChmod->isChecked());
	proceedBoolOption("ChrootEveryone", ChrootEveryone->isChecked());

	if(sbTrustedGID->text().toInt() != 0)
		writeFileOption(PUREFTPD_CONFIG_PATH, "TrustedGID", sbTrustedGID->text());
	else
		removeFileOption(PUREFTPD_CONFIG_PATH, "TrustedGID");

	if(spMinUID->text().toInt() != 0)
		writeFileOption(PUREFTPD_CONFIG_PATH, "MinUID", spMinUID->text());
	else
		removeFileOption(PUREFTPD_CONFIG_PATH, "MinUID");

	proceedBoolOption("AllowUserFXP", AllowUserFXP->isChecked());

	if(NoAnonymous->isChecked())
	{
		writeFileOption(PUREFTPD_CONFIG_PATH, "NoAnonymous", "yes");

		removeFileOption(PUREFTPD_CONFIG_PATH, "AllowAnonymousFXP");
		removeFileOption(PUREFTPD_CONFIG_PATH, "AnonymousCanCreateDirs");
		removeFileOption(PUREFTPD_CONFIG_PATH, "AnonymousCantUpload");
		removeFileOption(PUREFTPD_CONFIG_PATH, "AntiWarez");
		removeFileOption(PUREFTPD_CONFIG_PATH, "TrustedIP");
		removeFileOption(PUREFTPD_CONFIG_PATH, "AnonymousRatio");
		removeFileOption(PUREFTPD_CONFIG_PATH, "AnonymousBandwidth");
	}
	else
	{
		removeFileOption(PUREFTPD_CONFIG_PATH, "NoAnonymous");

		proceedBoolOption("AllowAnonymousFXP", AllowAnonymousFXP->isChecked());
		proceedBoolOption("AnonymousCanCreateDirs", AnonymousCanCreateDirs->isChecked());
		proceedBoolOption("AnonymousCantUpload", AnonymousCantUpload->isChecked());
		proceedBoolOption("AntiWarez", cbAntiWarez->isChecked());
		proceedStringOption("TrustedIP", leTrustedIP->text());
		proceedStringOption("AnonymousRatio", lnAnonymousRatio->text());

		s = sbAnonymousBandwidth->text().section(" ", 0, 0);
		proceedStringOption("AnonymousBandwidth", s);
	}

	// logging tab
	proceedBoolOption("LogPID", LogPID->isChecked());
	proceedBoolOption("VerboseLog", VerboseLog->isChecked());
	proceedStringOption("AltLog", leAltLog->text());
	proceedStringOption("SyslogFacility", leSyslogFacility->text());

	// Files & Directories tab
	proceedStringOption("LimitRecursion", leLimitRecursion->text());
	proceedStringOption("PerUserLimits", lePerUserLimits->text());
	proceedStringOption("Quota", leQuota->text());
	proceedStringOption("UserRatio", leUserRatio->text());
	proceedStringOption("MaxLoad", leMaxLoad->text());
	proceedStringOption("Umask", leUmask->text());
	proceedBoolOption("AutoRename", AutoRename->isChecked());
	proceedBoolOption("NoRename", NoRename->isChecked());
	proceedBoolOption("KeepAllFiles", KeepAllFiles->isChecked());
	proceedBoolOption("DisplayDotFiles", cbDisplayDotFiles->isChecked());
	proceedBoolOption("AllowDotFiles", cbAllowDotFiles->isChecked());
	proceedBoolOption("ProhibitDotFilesWrite", cbProhibitDotFilesWrite->isChecked());
	proceedBoolOption("ProhibitDotFilesRead", cbProhibitDotFilesRead->isChecked());

 	progression->setProgress(100);

	return TRUE;
}

bool DlgGPureftpdImpl::proceedBoolOption(QString optionName, bool state)
{
	bool b;
	b = FALSE;

	if(state)
		b = writeFileOption(PUREFTPD_CONFIG_PATH, optionName, "yes");
	else
		b = removeFileOption(PUREFTPD_CONFIG_PATH, optionName);

	return b;
}

bool DlgGPureftpdImpl::proceedStringOption(QString optionName, QString content)
{
	bool b;
	b = FALSE;

	if(content != QString::null && !content.isEmpty())
		b = writeFileOption(PUREFTPD_CONFIG_PATH, optionName, content);
	else
		b = removeFileOption(PUREFTPD_CONFIG_PATH, optionName);

	return b;
}

bool DlgGPureftpdImpl::writeFileOption(QString path, QString sfilename, QString content)
{
	QFile *f;
	bool bResult;
	QString sfile;

	bResult = FALSE;
	sfile = path + sfilename;

	removeFileOption(path, sfilename);

	f = new QFile(sfile);

	if (f->open(IO_WriteOnly))
	{
		QTextStream ts(f);
		ts << content;
		f->close();
		bResult = TRUE;
	}

	return bResult;
}

bool DlgGPureftpdImpl::removeFileOption(QString path, QString sfilename)
{
	bool bResult;
	bResult = TRUE;
	if(QFile::exists(path + sfilename))
		bResult = QFile::remove(path + sfilename);

	return bResult;
}

void DlgGPureftpdImpl::manageVirtualUsers()
{
	DlgVirtualUsersImpl *mvu = new DlgVirtualUsersImpl(this, NULL, TRUE);
	mvu->show();
}

int DlgGPureftpdImpl::messageBox(QString caption, QString text, QMessageBox::Icon icon, int button0/* = QMessageBox::Ok */, int button1/* = 0 */, int button2/* = 0 */)
{
	QMessageBox *mgb = new QMessageBox(caption, text,  icon, button0, button1, button2, this);
	mgb->setFont(this->font());
	return mgb->exec();
}

int DlgGPureftpdImpl::messageBox(QWidget *parent, QString caption, QString text, QMessageBox::Icon icon, int button0/* = QMessageBox::Ok */, int button1/* = 0 */, int button2/* = 0 */)
{
	QMessageBox *mgb = new QMessageBox(caption, text,  icon, button0, button1, button2, parent);
	mgb->setFont(parent->font());
	return mgb->exec();
}
