/*
    dlgpasswordimpl.cpp

    Copyright (c) 2006 by Xavier Facq

    gpureftpd (c) 2006 by the gpureftpd developers

    *************************************************************************
    *                                                                       *
    * This program is free software; you can redistribute it and/or modify  *
    * it under the terms of the GNU General Public License as published by  *
    * the Free Software Foundation; either version 2 of the License, or     *
    * (at your option) any later version.                                   *
    *                                                                       *
    *************************************************************************
*/


#include "dlgpasswordimpl.h"
#include "dlggpureftpdimpl.h"

#include <qlineedit.h>
#include <qpushbutton.h>

#define _XOPEN_SOURCE_
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <sys/types.h>
#include <signal.h>
#include <errno.h>
#include <stdarg.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

DlgPasswordImpl::DlgPasswordImpl(QWidget* parent, const char* name, bool modal, WFlags fl)
    :DlgPassword(parent, name, modal, fl)
{
	m_EncodedPassword = NULL;
	m_SomethingHaveChanged = FALSE;

	setFont(parent->font());

	connect(lePassword, SIGNAL(textChanged(const QString&)), this, SLOT(somethingChangeSomewhere(const QString&)));
	connect(leVerifyPassword, SIGNAL(textChanged(const QString&)), this, SLOT(somethingChangeSomewhere(const QString&)));

    connect(pbSave, SIGNAL(clicked()), this, SLOT(accept()));
    connect(pbCancel, SIGNAL(clicked()), this, SLOT(reject()));

	setMinimumSize(width(), height());
	setMaximumSize(width(), height());
}

void DlgPasswordImpl::somethingChangeSomewhere(const QString& newText)
{
	if(!m_SomethingHaveChanged) m_SomethingHaveChanged = TRUE;
}

void DlgPasswordImpl::setEncodedPassword(QString encodedPassword)
{
	if(encodedPassword != NULL && encodedPassword != QString::null && !encodedPassword.isEmpty())
	{
		m_EncodedPassword = new QString(encodedPassword);

		// put the first 8 caracteres of the encoded password in fields as fake password :-)
		lePassword->setText(encodedPassword.left(8));
		leVerifyPassword->setText(encodedPassword.left(8));
		m_SomethingHaveChanged = FALSE;
	}
}

QString* DlgPasswordImpl::getEncodedPassword()
{
	return m_EncodedPassword;
}

void DlgPasswordImpl::accept()
{
	if(lePassword->text().isEmpty() || leVerifyPassword->text().isEmpty())
		DlgGPureftpdImpl::messageBox(this, "Error", "You must give a password and verify it.", QMessageBox::Warning);
	else if(lePassword->text() != leVerifyPassword->text())
		DlgGPureftpdImpl::messageBox(this, "Error", "Password aren't the same.", QMessageBox::Warning);
	else
	{
		if(m_EncodedPassword == NULL || m_SomethingHaveChanged)
			m_EncodedPassword = new QString(encode(lePassword->text()));

		DlgPassword::accept();
	}
}

void DlgPasswordImpl::reject()
{
	if(m_SomethingHaveChanged && DlgGPureftpdImpl::messageBox(this, "Confirm quit", "Do you want to quit without saving your changes ?", QMessageBox::Question, QMessageBox::Yes, QMessageBox::No) == QMessageBox::No)
		return;

	DlgPassword::reject();
}

/*
They are crypto hashed with blowfish, md5, multiple-des
and simple des, in this order, according to what your system has support fort.
*/
QString DlgPasswordImpl::encode(QString pwd)
{
    static const char crcars[65] =
        "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789./";
    register const char *crypted;
    
    if ((crypted = (const char *)      /* Blowfish */
         crypt("test", "$2a$07$1234567890123456789012")) != NULL &&        
        strcmp(crypted, "$2a$07$123456789012345678901uKO4"
               "/IReKqBzRzT6YaajGvw20UBdHW7m") == 0) {
        char salt[] = "$2a$07$0000000000000000000000";        
        int c = 28;
        
        do {            
            c--;
            salt[c] = crcars[random() & 63];
        } while (c > 7);
        
        return (char *) crypt(pwd, salt);        
    } else if ((crypted = (const char *)    /* MD5 */
                crypt("test", "$1$12345678$")) != NULL &&
               strcmp(crypted, "$1$12345678$oEitTZYQtRHfNGmsFvTBA/") == 0) {
        char salt[] = "$1$00000000";
        int c = 10;
        
        do {            
            c--;
            salt[c] = crcars[random() & 63];
        } while (c > 3);
        
        return (char *) crypt(pwd, salt);
    } else if ((crypted = (const char *)    /* Extended DES */
                crypt("test", "_.../1234")) != NULL &&
               strcmp(crypted, "_.../1234PAPUVmqGzpU") == 0) {
        char salt[] = "_.../0000";
        int c = 8;
        
        do {
            c--;
            salt[c] = crcars[random() & 63];
        } while (c > 5);
        
        return (char *) crypt(pwd, salt);
    }
    /* Simple DES */
    {
        char salt[] = "00";
        
        salt[0] = crcars[random() & 63];
        salt[1] = crcars[random() & 63];
        
        return (char *) crypt(pwd, salt);        
    }    
}
