/*
    dlgvirtualusersimpl.cpp

    Copyright (c) 2006 by Xavier Facq

    gpureftpd (c) 2006 by the gpureftpd developers

    *************************************************************************
    *                                                                       *
    * This program is free software; you can redistribute it and/or modify  *
    * it under the terms of the GNU General Public License as published by  *
    * the Free Software Foundation; either version 2 of the License, or     *
    * (at your option) any later version.                                   *
    *                                                                       *
    *************************************************************************
*/


#include "global.h"
#include "virtualuser.h"
#include "dlgpasswordimpl.h"
#include "dlggpureftpdimpl.h"
#include "dlgvirtualusersimpl.h"

#include <qfile.h>
#include <qlistview.h>
#include <qspinbox.h>
#include <qobjectlist.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qprocess.h>

DlgVirtualUsersImpl::DlgVirtualUsersImpl(QWidget* parent, const char* name, bool modal, WFlags fl)
    : DlgVirtualUsers(parent, name, modal, fl)
{
	if(parent != NULL)
		setFont(parent->font());

	m_DetailsChanged = FALSE;
	m_SomethingHaveChanged = FALSE;
	m_VirtualUserList = new QPtrList<VirtualUser>();

	loadVirtualUsersFromFile();

	fillListView();

	QObject *obj;
	QObjectList *widgets = queryList();
	QObjectListIterator it(*widgets);
	
	while((obj = it.current()) != NULL)
	{
		QString className = obj->className();

		if(className == "QLineEdit" || className == "QComboBox")
		    connect(obj, SIGNAL(textChanged(const QString&)), this, SLOT(somethingChangeSomewhere(const QString&)));
		else if(className == "QSpinBox")
		    connect(obj, SIGNAL(valueChanged(const QString&)), this, SLOT(somethingChangeSomewhere(const QString&)));
 		else if(className == "QCheckBox" || className == "QRadioButton")
 		    connect(obj, SIGNAL(toggled(bool)), this, SLOT(somethingChangeSomewhere(bool)));

		++it;
	}

    connect(lvVirtualUserList, SIGNAL(clicked (QListViewItem *)), this, SLOT(showUserInfo(QListViewItem *)));
    connect(leLogin, SIGNAL(lostFocus()), this, SLOT(controlChangeInDetailsZone()));
    connect(pbAddUser, SIGNAL(clicked()), this, SLOT(addVirtualUser()));
    connect(pbRemoveUser, SIGNAL(clicked()), this, SLOT(removeVirtualUser()));
    connect(pbSave, SIGNAL(clicked()), this, SLOT(accept()));
    connect(pbCancel, SIGNAL(clicked()), this, SLOT(reject()));
    connect(pbPassword, SIGNAL(clicked()), this, SLOT(enterPassword()));

	setMinimumSize(width(), height());
	setMaximumSize(width(), height());
}

bool DlgVirtualUsersImpl::loadVirtualUsersFromFile()
{
	VirtualUser *vu;
	QString sfilename;

	sfilename = PUREFTPD_PATH + "pureftpd.passwd";

	if(QFile::exists(sfilename))
	{
		QFile f(sfilename);
		if (f.open( IO_ReadOnly))
		{
			QTextStream ts( &f );

			while(!ts.atEnd())
			{
				vu = new VirtualUser(ts.readLine());
				m_VirtualUserList->append(vu);
			}

			f.close();
		}
	}

	return TRUE;
}

void DlgVirtualUsersImpl::fillListView()
{
	for (VirtualUser *vu = m_VirtualUserList->first(); vu; vu = m_VirtualUserList->next())
	{
		QListViewItem qlv = new QListViewItem(lvVirtualUserList, vu->getLogin(), vu->getFullName(), vu->getUID(), vu->getGID(), vu->getDirectory());
		lvVirtualUserList->insertItem(&qlv);
	}
}

VirtualUser* DlgVirtualUsersImpl::getByLogin(QString login)
{
	VirtualUser *vu;

	for (vu = m_VirtualUserList->first(); vu; vu = m_VirtualUserList->next())
		if(vu->getLogin() == login)	break;

	return vu;
}

void DlgVirtualUsersImpl::showUserInfo(QListViewItem *item)
{
	controlChangeInDetailsZone();

	if(item == NULL)
		clearDetailsZone();
	else
	{
		VirtualUser *vu;
		vu = getByLogin(item->text(0));
		synchronizeDetailsFromList(vu);
		leLogin->setReadOnly(TRUE);
		m_DetailsChanged = FALSE;
	}
}

/* update the detail zone from the selected item in the list  */
void DlgVirtualUsersImpl::synchronizeDetailsFromList(VirtualUser *vu)
{
	if(vu != NULL)
	{
		leLogin->setText(vu->getLogin());
		leFullName->setText(vu->getFullName());
		sbUID->setValue(vu->getUID().toInt());
		sbGID->setValue(vu->getGID().toInt());
		leDirectory->setText(vu->getDirectory());
		sbDownloadBW->setValue(vu->getdwlBandwidth().toInt());
		sbUploadBW->setValue(vu->getUploadBandwidth().toInt());
		sbMaxFiles->setValue(vu->getMaxFiles().toInt());
		sbMaxSize->setValue(vu->getMaxSize().toInt());
		leRatio->setText(vu->getRatio());
		leAllowedLocalIPs->setText(vu->getAllowedlocalIPs());
		leDeniedLocalIPs->setText(vu->getDeniedLocalIPs());
		leAllowedClientIPs->setText(vu->getAllowedClientIPs());
		leDeniedClientIPs->setText(vu->getDeniedClientIPs());
		leTimeRestrictions->setText(vu->getTimeRestrictions());
		sbMaxSimSessions->setValue(vu->getMaxSimSessions().toInt());
	}
}

/* update the selected item in the list from the detail zone */
void DlgVirtualUsersImpl::synchronizeListFromDetails(VirtualUser *vu)
{
	if(vu == NULL)
	{
		vu = new VirtualUser();
		m_VirtualUserList->append(vu);
	}

	vu->setLogin(leLogin->text());
	vu->setFullName(leFullName->text());
	vu->setUID(sbUID->text());
	vu->setGID(sbGID->text());
	vu->setDirectory(leDirectory->text());
	vu->setdwlBandwidth(sbDownloadBW->value() == 0 ? "" : sbDownloadBW->text().section(" ", 0, 0));
	vu->setUploadBandwidth(sbUploadBW->value() == 0 ? "" : sbUploadBW->text().section(" ", 0, 0));
	vu->setMaxFiles(sbMaxFiles->value() == 0 ? "" : sbMaxFiles->text());
	vu->setMaxSize(sbMaxSize->value() == 0 ? "" : sbMaxSize->text().section(" ", 0, 0));
	vu->setRatio(leRatio->text());
	vu->setAllowedlocalIPs(leAllowedLocalIPs->text());
	vu->setDeniedLocalIPs(leDeniedLocalIPs->text());
	vu->setAllowedClientIPs(leAllowedClientIPs->text());
	vu->setDeniedClientIPs(leDeniedClientIPs->text());
	vu->setTimeRestrictions(leTimeRestrictions->text());

	vu->setMaxSimSessions(sbMaxSimSessions->value() == 0 ? "" : sbMaxSimSessions->text());

	QListViewItem *item = lvVirtualUserList->findItem(leLogin->text(), 0);

	if(item == NULL)
		item = new QListViewItem(lvVirtualUserList, vu->getLogin(), vu->getFullName(), vu->getUID(), vu->getGID(), vu->getDirectory());

	item->setText(1, leFullName->text());
	item->setText(2, sbUID->text());
	item->setText(3, sbGID->text());
	item->setText(4, leDirectory->text());
}

void DlgVirtualUsersImpl::addVirtualUser()
{
	controlChangeInDetailsZone();

	clearDetailsZone();
}

void DlgVirtualUsersImpl::removeVirtualUser()
{
	VirtualUser *vu;

	if(lvVirtualUserList->selectedItem() != 0)
	{
		if(DlgGPureftpdImpl::messageBox(this, "Confirm delete", "Do you really want to remove this virtual user ?", QMessageBox::Question, QMessageBox::Yes, QMessageBox::No) == QMessageBox::Yes)
		{
			clearDetailsZone();

			vu = getByLogin(lvVirtualUserList->selectedItem()->text(0));
			m_VirtualUserList->remove(vu);

			lvVirtualUserList->takeItem (lvVirtualUserList->selectedItem());

			m_SomethingHaveChanged = TRUE;
		}
	}
}

void DlgVirtualUsersImpl::accept()
{
	controlChangeInDetailsZone();

	if(checkUncompletUsersData())
	{
		if(m_SomethingHaveChanged && DlgGPureftpdImpl::messageBox(this, "Confirm save", "Do you want to save your changes ?", QMessageBox::Question, QMessageBox::Yes, QMessageBox::No) == QMessageBox::No)
			return;

		writePureftpdPasswdFile();

		rebuildPureftpdPdbDatabase();

		DlgVirtualUsers::accept();
	}
}

void DlgVirtualUsersImpl::reject()
{
	controlChangeInDetailsZone();

	if(m_SomethingHaveChanged && DlgGPureftpdImpl::messageBox(this, "Confirm quit", "Do you want to quit without saving your changes ?", QMessageBox::Question, QMessageBox::Yes, QMessageBox::No) == QMessageBox::No)
		return;

	DlgVirtualUsers::reject();
}

void DlgVirtualUsersImpl::controlChangeInDetailsZone()
{
	if(m_DetailsChanged)
	{
		if(leLogin->text() != QString::null && !leLogin->text().isEmpty())
		{
			synchronizeListFromDetails(getByLogin(leLogin->text()));
			leLogin->setReadOnly(TRUE);
			m_SomethingHaveChanged = TRUE;
		}

		m_DetailsChanged = FALSE;
	}
}

bool DlgVirtualUsersImpl::checkUncompletUsersData()
{
	bool b;
	QString msg;
	VirtualUser *vu;

	b = TRUE;
	msg = QString::null;

	for (vu = m_VirtualUserList->first(); vu; vu = m_VirtualUserList->next())
	{
		if(vu->getLogin() == QString::null || vu->getLogin().isEmpty())
		{
			msg = "You cannot create a virtual user with an empty login.";
			break;
		}
		else if(vu->getDirectory() == QString::null || vu->getDirectory().isEmpty())
		{
			msg = "You must define a directory for the user " + vu->getLogin();
			break;
		}
		else if(vu->getPassword() == QString::null || vu->getPassword().isEmpty())
		{
			msg = "You must define a password for the user " + vu->getLogin();
			break;
		}
	}

	if(msg != QString::null)
	{
		b = FALSE;
		DlgGPureftpdImpl::messageBox(this, "Error", msg, QMessageBox::Warning);
	}

	return b;
}

void DlgVirtualUsersImpl::clearDetailsZone()
{
	leLogin->clear();
	leFullName->clear();
	sbUID->setValue(0);
	sbGID->setValue(0);
	leDirectory->clear();
	sbDownloadBW->setValue(0);
	sbUploadBW->setValue(0);
	sbMaxFiles->setValue(0);
	sbMaxSize->setValue(0);
	leRatio->clear();
	leAllowedLocalIPs->clear();
	leDeniedLocalIPs->clear();
	leAllowedClientIPs->clear();
	leDeniedClientIPs->clear();
	leTimeRestrictions->clear();
	sbMaxSimSessions->setValue(0);

	leLogin->setReadOnly(FALSE);
}

void DlgVirtualUsersImpl::somethingChangeSomewhere(const QString& newText)
{
	if(!m_DetailsChanged) m_DetailsChanged = TRUE;
}

void DlgVirtualUsersImpl::somethingChangeSomewhere(bool b)
{
	if(!m_DetailsChanged) m_DetailsChanged = TRUE;
}

void DlgVirtualUsersImpl::writePureftpdPasswdFile()
{
	QFile *f;
	VirtualUser *vu;
	QString sfilename;

	sfilename = PUREFTPD_PATH + "pureftpd.passwd";

	f = new QFile(sfilename);
	QTextStream ts(f);

	if (f->open(IO_WriteOnly))
	
	for (vu = m_VirtualUserList->first(); vu; vu = m_VirtualUserList->next())
		ts << vu->toString();

	f->close();
}

void DlgVirtualUsersImpl::rebuildPureftpdPdbDatabase()
{
	QProcess *proc;
	proc = new QProcess(this);
	proc->addArgument("/usr/bin/pure-pw");
	proc->addArgument("mkdb");
	proc->start();
}

void DlgVirtualUsersImpl::enterPassword()
{
	VirtualUser *vu;
	vu = getByLogin(leLogin->text());

	if(vu != NULL)
	{
		DlgPasswordImpl *dpi;
		dpi = new DlgPasswordImpl(this, NULL, TRUE);
		dpi->setEncodedPassword(vu->getPassword());

		if(dpi->exec() == QDialog::Accepted)
		{
			vu->setPassword(dpi->getEncodedPassword()->utf8());
			m_DetailsChanged = TRUE;
		}
	}
}
