/*
    main.cpp

    Copyright (c) 2006 by Xavier Facq

    gpureftpd (c) 2006 by the gpureftpd developers

    *************************************************************************
    *                                                                       *
    * This program is free software; you can redistribute it and/or modify  *
    * it under the terms of the GNU General Public License as published by  *
    * the Free Software Foundation; either version 2 of the License, or     *
    * (at your option) any later version.                                   *
    *                                                                       *
    *************************************************************************
*/


#include <qapplication.h>

#include "global.h"
#include "dlggpureftpdimpl.h"
#include "dlgvirtualusersimpl.h"

void printHelp();

int main( int argc, char ** argv )
{
	bool bStartDialog, bVirtualMode;
    QApplication a( argc, argv );

	bStartDialog = TRUE;
	bVirtualMode = FALSE;

	for(int n = argc; n >= 0; n--)
	{
		QString s = QString(argv[n]);

		if(s == "-u" || s == "--user")
		{
			bVirtualMode = TRUE;
			break;
		}
		else if(s == "-v" || s == "--version")
		{
			bStartDialog = FALSE;
			printf("gpureftpd version " + QString(GPUREFTPD_VERSION) + "\n");
			break;
		}
		else if(s == "-h" || s == "--help" || s.startsWith("-"))
		{
			bStartDialog = FALSE;
			printHelp();
			break;
		}
	}

	if(bStartDialog)
	{
		if(bVirtualMode)
		{
			DlgVirtualUsersImpl *mvu = new DlgVirtualUsersImpl();
			mvu->show();
		}
		else
		{
			DlgGPureftpdImpl *scp = new DlgGPureftpdImpl();
			scp->show();
		}

		a.connect( &a, SIGNAL(lastWindowClosed()), &a, SLOT(quit()) );
		return a.exec();
	}
}

void printHelp()
{
		printf("Usage : gpureftp [options]\n");
		printf("Options:\n");
		printf("-u, --user                      only start with the virtual users interface.\n");
		printf("-h, --help                      show this help.\n");
		printf("-v, --version                   shown the gpureftpd version and quit.\n");
		printf("\n");
}